@@ -1,7 +1,7 @@
 import inspect
 from collections import OrderedDict
 from enum import IntEnum
-from typing import Any, Mapping, Sequence, Type, Union  # noqa
+from typing import Any, List, Mapping, Set, Type, Union
 
 from .exceptions import ConfigError, Error, type_display
 from .validators import NoneType, find_validators, not_none_validator
@@ -15,18 +15,20 @@ class ValidatorSignature(IntEnum):
 
 class Shape(IntEnum):
     SINGLETON = 1
-    SEQUENCE = 2
-    MAPPING = 3
+    LIST = 2
+    SET = 3
+    MAPPING = 4
 
 
 class Field:
-    __slots__ = ('type_', 'key_type_', 'sub_fields', 'key_field', 'validators', 'default',
-                 'required', 'name', 'description', 'info', 'validate_always', 'allow_none', 'shape', 'multipart')
+    __slots__ = ('type_', 'key_type_', 'sub_fields', 'key_field', 'validators', 'default', 'required',
+                 'name', 'alias', 'description', 'info', 'validate_always', 'allow_none', 'shape', 'multipart')
 
     def __init__(
             self, *,
-            name: str=None,
+            name: str,
             type_: Type,
+            alias: str=None,
             class_validators: dict=None,
             default: Any=None,
             required: bool=False,
@@ -34,6 +36,7 @@ class Field:
             description: str=None):
 
         self.name: str = name
+        self.alias: str = alias or name
         self.type_: type = type_
         self.key_type_: type = None
         self.validate_always: bool = getattr(self.type_, 'validate_always', False)
@@ -50,16 +53,22 @@ class Field:
         self._prepare(class_validators or {})
 
     @classmethod
-    def infer(cls, *, name, value, annotation, class_validators):
+    def infer(cls, *, name, value, annotation, class_validators, field_config):
         required = value == Ellipsis
         return cls(
+            name=name,
             type_=annotation,
+            alias=field_config and field_config.get('alias'),
             class_validators=class_validators,
             default=None if required else value,
             required=required,
-            name=name
+            description=field_config and field_config.get('description'),
         )
 
+    @property
+    def alt_alias(self):
+        return self.name != self.alias
+
     def _prepare(self, class_validators):
         if self.default is not None and self.type_ is None:
             self.type_ = type(self.default)
@@ -94,6 +103,7 @@ class Field:
         # typing interface is horrible, we have to do some ugly checks
         origin = getattr(self.type_, '__origin__', None)
         if origin is None:
+            # field is not "typing" object eg. Union, Dict, List etc.
             return
 
         if origin is Union:
@@ -112,9 +122,12 @@ class Field:
                 name=f'{self.name}_{type_display(t)}'
             ) for t in types_]
             self.multipart = True
-        elif issubclass(origin, Sequence):
+        elif issubclass(origin, List):
             self.type_ = self.type_.__args__[0]
-            self.shape = Shape.SEQUENCE
+            self.shape = Shape.LIST
+        elif issubclass(origin, Set):
+            self.type_ = self.type_.__args__[0]
+            self.shape = Shape.SET
         else:
             assert issubclass(origin, Mapping)
             self.key_type_ = self.type_.__args__[0]
@@ -129,9 +142,9 @@ class Field:
                 name=f'key_{self.name}'
             )
 
-        if self.sub_fields is None and getattr(self.type_, '__origin__', None):
+        if self.sub_fields is None and getattr(self.type_, '__origin__', False):
             self.multipart = True
-            self.sub_fields = self.sub_fields or [Field(
+            self.sub_fields = [Field(
                 type_=self.type_,
                 class_validators=class_validators,
                 default=self.default,
@@ -164,10 +177,14 @@ class Field:
 
         if self.shape is Shape.SINGLETON:
             return self._validate_singleton(v, model, index)
-        elif self.shape is Shape.SEQUENCE:
-            return self._validate_sequence(v, model)
-        else:
+        elif self.shape is Shape.MAPPING:
             return self._validate_mapping(v, model)
+        else:
+            # list or set
+            result, errors = self._validate_sequence(v, model)
+            if not errors and self.shape is Shape.SET:
+                return set(result), errors
+            return result, errors
 
     def _validate_sequence(self, v, model):
         result, errors = [], []
