@@ -14,7 +14,8 @@ class BaseConfig:
     raise_exception = True
     validate_all = False
     ignore_extra = True
-    # TODO allow extra
+    allow_extra = False
+    fields = None
 
 
 def inherit_config(self_config, parent_config) -> BaseConfig:
@@ -47,14 +48,19 @@ class MetaModel(type):
         class_validators = {n: f for n, f in namespace.items()
                             if n.startswith('validate_') and isinstance(f, FunctionType)}
 
+        config_fields = config.fields or {}
         for var_name, value in namespace.items():
             if var_name.startswith('_') or isinstance(value, TYPE_BLACKLIST):
                 continue
+            field_config = config_fields.get(var_name)
+            if isinstance(field_config, str):
+                field_config = {'alias': field_config}
             field = Field.infer(
                 name=var_name,
                 value=value,
                 annotation=annotations and annotations.get(var_name),
                 class_validators=class_validators,
+                field_config=field_config,
             )
             fields[var_name] = field
         namespace.update(
@@ -78,9 +84,20 @@ class BaseModel(metaclass=MetaModel):
         self.__errors__ = OrderedDict()
         self._process_values(values)
 
+    def setattr(self, name, value):
+        """
+        alternative to setattr() which checks the field exists and updates __values__.
+
+        This exists instead of overriding __setattr__ as that seems to cause a universal 10% slow down.
+        """
+        if not self.config.allow_extra and name not in self.__fields__:
+            raise ValueError(f'"{self.__class__.__name__}" object has no field "{name}"')
+        setattr(self, name, value)
+        self.__values__[name] = value
+
     @property
     def values(self):
-        return self.__values__
+        return dict(self)
 
     @property
     def fields(self):
@@ -101,25 +118,32 @@ class BaseModel(metaclass=MetaModel):
 
     def _process_values(self, values):
         for name, field in self.__fields__.items():
-            value = values.get(name, MISSING)
-            self._process_value(name, field, value)
+            value = values.get(field.alias, MISSING)
+            self._process_value(name, field.alias, field, value)
 
-        if not self.config.ignore_extra:
-            extra = values.keys() - self.__fields__.keys()
+        if not self.config.ignore_extra or self.config.allow_extra:
+            extra = values.keys() - {f.alias for f in self.__fields__.values()}
             if extra:
-                for field in sorted(extra):
-                    self.__errors__[field] = EXTRA_ERROR
+                if self.config.allow_extra:
+                    for field in extra:
+                        value = values[field]
+                        self.__values__[field] = value
+                        setattr(self, field, value)
+                else:
+                    # config.ignore_extra is False
+                    for field in sorted(extra):
+                        self.__errors__[field] = EXTRA_ERROR
 
         if self.config.raise_exception and self.__errors__:
             raise ValidationError(self.__errors__)
 
-    def _process_value(self, name, field, value):
+    def _process_value(self, name, alias, field, value):
         if value is MISSING:
             if self.config.validate_all or field.validate_always:
                 value = field.default
             else:
                 if field.required:
-                    self.__errors__[name] = MISSING_ERROR
+                    self.__errors__[alias] = MISSING_ERROR
                 else:
                     self.__values__[name] = field.default
                     # could skip this if the attributes equals field.default, would it be quicker?
@@ -128,13 +152,29 @@ class BaseModel(metaclass=MetaModel):
 
         value, errors = field.validate(value, self)
         if errors:
-            self.__errors__[name] = errors
+            self.__errors__[alias] = errors
         self.__values__[name] = value
         setattr(self, name, value)
 
+    @classmethod
+    def _get_value(cls, v):
+        if isinstance(v, BaseModel):
+            return v.values
+        elif isinstance(v, list):
+            return [cls._get_value(v_) for v_ in v]
+        elif isinstance(v, dict):
+            return {k_: cls._get_value(v_) for k_, v_ in v.items()}
+        elif isinstance(v, set):
+            return {cls._get_value(v_) for v_ in v}
+        elif isinstance(v, tuple):
+            return tuple(cls._get_value(v_) for v_ in v)
+        else:
+            return v
+
     def __iter__(self):
         # so `dict(model)` works
-        yield from self.__values__.items()
+        for k, v in self.__values__.items():
+            yield k, self._get_value(v)
 
     def __eq__(self, other):
         if isinstance(other, BaseModel):
